/*KSlacky InfoPkg is a package information tool that provide
the list of packages installe on your Slackware system
Copyright (C) 2008  Giovanni Santostefano

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.*/

#include <KLocale>
#include <KMessageBox>
#include <KFileDialog>
#include <QDir>
#include <QStringList>
#include <QTextStream>
#include <QRegExp>

#include <KMessageBox> //new
#include <KIO/NetAccess> //new
#include <KSaveFile> //new


#include <unistd.h>
#include <stdio.h>
#include <iostream>
#include <string>

#include "mainwindow.h"

using namespace std;

MainWindow::MainWindow(QWidget *parent)
: QWidget(parent)
{
	searchindex=0;
	QWidget *w = new QWidget(this);
	ui.setupUi(w);
    connect( ui.pkglist, SIGNAL( itemSelectionChanged() ),
             this, SLOT( PackageListIndexChange() ) );
	connect( ui.aboutbutton, SIGNAL( clicked() ),
             this, SLOT( AboutButtonPressed() ) );
	connect( ui.exitbutton, SIGNAL( clicked() ),
             this, SLOT( ExitButtonPressed() ) );
             
	connect( ui.searchbutton, SIGNAL( clicked() ),
             this, SLOT( SearchButtonPressed() ) );
	connect( ui.searchnextbutton, SIGNAL( clicked() ),
             this, SLOT( SearchNextButtonPressed() ) );
             
	connect( ui.installbutton, SIGNAL( clicked() ),
             this, SLOT( InstallPkgButtonPressed() ) );
	connect( ui.upgradebutton, SIGNAL( clicked() ),
             this, SLOT( UpgradePkgButtonPressed() ) );    
	connect( ui.removebutton, SIGNAL( clicked() ),
             this, SLOT( RemovePkgButtonPressed() ) );                      
	connect( ui.refreshbutton, SIGNAL( clicked() ),
             this, SLOT( RefreshButtonPressed() ) );         
    
    PKGLoading();
}

MainWindow::~MainWindow()
{
}

void MainWindow::PKGLoading()
{
	QDir dirman=QDir("/var/log/packages");
	
	ui.lcdNumber->display((int)dirman.count());
	
	dirman.setFilter(QDir::Files | QDir::Hidden | QDir::NoSymLinks);
    dirman.setSorting(QDir::Name);

    QFileInfoList list = dirman.entryInfoList();
	
	ui.pkglist->clear();
	for(int i=0; i<list.size(); ++i)
	{
		QFileInfo fileinfo = list.at(i);
		ui.pkglist->addItem(fileinfo.fileName());
	}
	
	ui.progressbar->setMaximum(list.size());
	ui.pkglist->setCurrentRow(0);
	ShowFileInfo(0);
}

void MainWindow::PackageListIndexChange()
{
	ShowFileInfo(ui.pkglist->currentRow());
}

void MainWindow::ExitButtonPressed()
{
	exit(0);
}

void MainWindow::SearchButtonPressed()
{
	//zeroing the index
	searchindex=0;
	
	//generating the regular expression
	QRegExp rx(ui.expsearchline->text());
	rx.setPatternSyntax(QRegExp::Wildcard);
	
	for(int i=0; i<ui.pkglist->count(); ++i)
	{
		QString s=ui.pkglist->item(i)->text();
		ui.progressbar->setValue(i);
		if(rx.exactMatch(s))
		{
		
			ui.pkglist->setCurrentRow(i);
			ShowFileInfo(i);
			return;
		}
		++searchindex;
	}
	
	ui.pkgdesc->setPlainText("Package not found");
}

void MainWindow::SearchNextButtonPressed()
{
	//generating the regular expression
	QRegExp rx(ui.expsearchline->text());
	rx.setPatternSyntax(QRegExp::Wildcard);
	
	for(int i=++searchindex; i<ui.pkglist->count(); ++i)
	{
		QString s=ui.pkglist->item(i)->text();
		ui.progressbar->setValue(i);
		if(rx.exactMatch(s))
		{
			ui.pkglist->setCurrentRow(i);
			ShowFileInfo(i);
			return;
		}
		++searchindex;
	}
	
	ui.pkgdesc->setPlainText("Package not found");
}

void MainWindow::AboutButtonPressed()
{
	KMessageBox::information( this, 
    i18n("KSlacky InfoPkg is a Slackware information tool\
    that shows info on installed packages\
    \nDeveloped by Giovanni Santostefano\
    \nhttp://santostefanogiovanni.blogspot.com" ),
                              i18n( "About KSlacky InfoPkg" ) );
}

void MainWindow::ShowFileInfo(int index)
{
	QString filepath="/var/log/packages/";
	filepath+=ui.pkglist->item(index)->text();
	
	//file opening and reading the content
	QFile file(filepath);
	file.open(QIODevice::ReadOnly);
	QString content=QTextStream(&file).readAll();
	
	//cut off the file list info
	QStringList liststrings = content.split("FILE LIST");
	if(liststrings.empty())
	{
		ui.pkgdesc->setPlainText("Can't retreive package informations");
		ui.pkgfiles->setPlainText("Can't retreive package file list");
	}
	else
	{
		ui.pkgdesc->setPlainText(liststrings.at(0));
		ui.pkgfiles->setPlainText(liststrings.at(1));
	}
}


void MainWindow::InstallPkgButtonPressed()
{
	//open a openfile dialog to select the package to open
	QString command="su -c installpkg\\ ";
	QString filename = KFileDialog::getOpenFileName(KUrl(),"*.tgz");
	
	if(filename.isEmpty()) return;
	
	command+=filename+"\0";
	pid_t pid=fork();
	
	if(pid==0)
	{
		string s=command.toStdString();
		execl("/usr/bin/xterm","xterm","-e",s.c_str(),(char*)0);
	}
}

void MainWindow::UpgradePkgButtonPressed()
{
	//open a openfile dialog to select the package to open
	QString command="su -c upgradepkg\\ ";
	QString filename = KFileDialog::getOpenFileName(KUrl(),"*.tgz");
	command+=filename+"\0";
	
	if(filename.isEmpty()) return;
	
	pid_t pid=fork();
	
	if(pid==0)
	{
		string s=command.toStdString();
		execl("/usr/bin/xterm","xterm","-e",s.c_str(),(char*)0);
	}
}

void MainWindow::RemovePkgButtonPressed()
{
	QString packpath="/var/log/packages/";
	packpath+=ui.pkglist->item(ui.pkglist->currentRow())->text();
	QString question=i18n("Are you sure to remove: ");
	question+=packpath;
	
	int res=KMessageBox::questionYesNo(this,question,i18n("Remove Package"));
	if(res==KMessageBox::Yes)
	{
	
		//ui.pkglist->removeItemWidget( 
			//		ui.pkglist->item(ui.pkglist->currentRow()) );
		ui.pkglist->setCurrentRow(0);
		
		QString command="su -c removepkg\\ ";
		command+=packpath+"\0";
		pid_t pid=fork();
	
		if(pid==0)
		{
			string s=command.toStdString();
			execl("/usr/bin/xterm","xterm","-e",s.c_str(),(char*)0);
		}
	}
}

void MainWindow::RefreshButtonPressed()
{
	PKGLoading();
}

//#include "mainwindow.moc"
